@@ -1,5 +1,4 @@
-from mpmath.libmpf import *
-from mpmath.libelefun import *
+from mpmath.libmp import *
 from mpmath import *
 import random
 import time
@@ -60,6 +59,10 @@ def test_constants():
             assert mertens == mpf(tmertens)
             assert twinprime == mpf(ttwinprime)
     mp.dps = 15
+    assert pi >= -1
+    assert pi > 2
+    assert pi > 3
+    assert pi < 4
 
 def test_exact_sqrts():
     for i in range(20000):
@@ -455,7 +458,7 @@ def test_aliases():
     assert degrees(5.6) == 5.6 / degree
     assert radians(5.6) == 5.6 * degree
     assert power(-1,0.5) == j
-    assert modf(25,7) == 4.0 and isinstance(modf(25,7), mpf)
+    assert fmod(25,7) == 4.0 and isinstance(fmod(25,7), mpf)
 
 def test_arg_sign():
     assert arg(3) == 0
@@ -514,7 +517,7 @@ def test_arange():
 def test_linspace():
     assert linspace(2, 9, 7) == [mpf('2.0'), mpf('3.166666666666667'),
         mpf('4.3333333333333339'), mpf('5.5'), mpf('6.666666666666667'),
-        mpf('7.8333333333333339'), mpf('9.0')] == linspace(mpi(2, 9), 7)
+        mpf('7.8333333333333339'), mpf('9.0')]
     assert linspace(2, 9, 7, endpoint=0) == [mpf('2.0'), mpf('3.0'), mpf('4.0'),
         mpf('5.0'), mpf('6.0'), mpf('7.0'), mpf('8.0')]
     assert linspace(2, 7, 1) == [mpf(2)]
@@ -744,6 +747,17 @@ def test_cospi_sinpi():
     assert cospi(1e-15) == 1
     assert cospi(1e-15, rounding='d') < 1
 
+def test_expj():
+    assert expj(0) == 1
+    assert expj(1).ae(exp(j))
+    assert expj(j).ae(exp(-1))
+    assert expj(1+j).ae(exp(j*(1+j)))
+    assert expjpi(0) == 1
+    assert expjpi(1).ae(exp(j*pi))
+    assert expjpi(j).ae(exp(-pi))
+    assert expjpi(1+j).ae(exp(j*pi*(1+j)))
+    assert expjpi(-10**15 * j).ae('2.22579818340535731e+1364376353841841')
+
 def test_sinc():
     assert sinc(0) == sincpi(0) == 1
     assert sinc(inf) == sincpi(inf) == 0
