@@ -463,6 +463,8 @@ def test_hyper_1f1():
     assert (hyp1f1(2,3,1e10j)*10**10).ae(-0.97501205020039745852 - 1.7462392454512132074j)
     # Shouldn't use asymptotic expansion
     assert hyp1f1(-2, 1, 10000).ae(49980001)
+    # Bug
+    assert hyp1f1(1j,fraction(1,3),0.415-69.739j).ae(25.857588206024346592 + 15.738060264515292063j)
 
 def test_hyper_2f1():
     mp.dps = 15
@@ -851,6 +853,7 @@ def test_gammainc():
     assert gammainc(1,b=1).ae(0.6321205588285576784)
     assert gammainc(3,2,2) == 0
     assert gammainc(2,3+j,3-j).ae(-0.28135485191849314194j)
+    assert gammainc(4+0j,1).ae(5.8860710587430771455)
     # Regularized upper gamma
     assert isnan(gammainc(0, 0, regularized=True))
     assert gammainc(-1, 0, regularized=True) == inf
@@ -1060,6 +1063,9 @@ def test_erf():
     assert erfc(inf) == 0
     assert isnan(erfc(nan))
     assert (erfc(10**4)*mpf(10)**43429453).ae('3.63998738656420')
+    assert erf(8+9j).ae(-1072004.2525062051158 + 364149.91954310255423j)
+    assert erfc(8+9j).ae(1072005.2525062051158 - 364149.91954310255423j)
+    assert erfc(-8-9j).ae(-1072003.2525062051158 + 364149.91954310255423j)
     mp.dps = 50
     # This one does not use the asymptotic series
     assert (erfc(10)*10**45).ae('2.0884875837625447570007862949577886115608181193212')
@@ -1270,3 +1276,7 @@ def test_spherharm():
     assert spherharm(3,0,0,1).ae(0.74635266518023078283)
     assert spherharm(3,-2,0,1) == 0
     assert spherharm(3,-2,1,1).ae(-0.16270707338254028971 - 0.35552144137546777097j)
+
+def test_qfunctions():
+    mp.dps = 15
+    assert qp(2,3,100).ae('2.7291482267247332183e2391')
