@@ -1,5 +1,6 @@
 from mpmath import *
-from mpmath.optimization import *
+from mpmath.calculus.optimization import Secant, Muller, Bisection, Illinois, \
+    Pegasus, Anderson, Ridder, ANewton, Newton, MNewton, MDNewton
 
 def test_findroot():
     # old tests, assuming secant
@@ -20,9 +21,12 @@ def test_findroot():
         assert abs(f(x)) < eps
     # test types
     f = lambda x: (x - 2)**2
-    assert isinstance(findroot(f, 1, force_type=mpf, tol=1e-10), mpf)
-    assert isinstance(findroot(f, 1., force_type=None, tol=1e-10), float)
-    assert isinstance(findroot(f, 1, force_type=complex, tol=1e-10), complex)
+
+    #assert isinstance(findroot(f, 1, force_type=mpf, tol=1e-10), mpf)
+    #assert isinstance(findroot(f, 1., force_type=None, tol=1e-10), float)
+    #assert isinstance(findroot(f, 1, force_type=complex, tol=1e-10), complex)
+    assert isinstance(fp.findroot(f, 1, tol=1e-10), float)
+    assert isinstance(fp.findroot(f, 1+0j, tol=1e-10), complex)
 
 def test_mnewton():
     f = lambda x: polyval([1,3,3,1],x)
@@ -48,7 +52,7 @@ def test_multidimensional():
     def f(*x):
         return [3*x[0]**2-2*x[1]**2-1, x[0]**2-2*x[0]+x[1]**2+2*x[1]-8]
     assert mnorm(jacobian(f, (1,-2)) - matrix([[6,8],[0,-2]]),1) < 1.e-7
-    for x, error in MDNewton(f, (1,-2), verbose=0,
+    for x, error in MDNewton(mp, f, (1,-2), verbose=0,
                              norm=lambda x: norm(x, inf)):
         pass
     assert norm(f(*x), 2) < 1e-14
